﻿local addonName, addon = ...
local L = LibStub("AceLocale-3.0"):GetLocale("ExPack");
local ExPack = ExPack;

--[[
	https://www.wowace.com/projects/ace3/pages/ace-config-3-0-options-tables
	
	Using AceConfig-3.0
	Creating an options table and handlers
	Changes - v0.2.0
		Enabled = true,
		ShowAll = true,
		ShowReagents = true,
		ErrorReporting = true,
		DebugTracing = false,
	Changes - v0.2.1
		ShowBanks = true,
	Changes - v0.2.3 - REJECTED
		RightAlignment = false,
		RightAlignment = {
			type = "toggle",
			name = "Text right alignment",
			desc = "Align the text to the right",
			width = "full",
			get = function(info) return ExPack.db.profile.RightAlignment; end,
			set = function(info, v) ExPack.db.profile.RightAlignment = v; end,
			order = 2
		},
	Changes - v0.2.4
		ShowBorder = false
	Changes - v0.2.5
		(no changes)
	Changes - v0.3.0
		(no changes)
	Changes - v0.3.1
		UseLibItemInfo = false,
	Changes - v0.3.2
		(no changes)
	Changes - v0.3.3
		[add]
		ShowConsumable = true,
		ShowTradegoods = true,
		ShowJunk = true,
		ShowQuestitem = false,
		ShowArmory = false,
		[mod]
		ShowAll = false,
		UseLibItemInfo = true,
	Changes - v0.3.4
		[add]
		AnyExpansion = true,
			CurrentExpack = true,
			OldExpacks = true,
	Changes - v0.3.5
		[add]
		minimap.hide
		[add]
		visual.Font = NumberFontNormalSmall
		visual.Position = TOPLEFT
		
	L["ExPack by |cFF66ccffGhordian |rOptions"]
	
]]--

ExPack.Fonts = {
    HighlightSmall = "GameFontHighlightSmall",
    Normal = "GameFontNormalOutline",
    Large = "GameFontNormalLargeOutline",
    Huge = "GameFontNormalHugeOutline",
    NumberNormal = "NumberFontNormal",
    NumberNormalSmall = "NumberFontNormalSmall",
}
--[[
    TOPLEFT = {2, -2},
    TOPRIGHT = {-2, -2},
    BOTTOMLEFT = {2, 2},
    BOTTOMRIGHT = {-2, 2},
    BOTTOM = {0, 2},
    TOP = {0, -2},
    LEFT = {2, 0},
    RIGHT = {-2, 0},
    CENTER = {0, 0},
]]--
ExPack.PositionOffsets = {
    TOPLEFT = {-2, 0},
    TOPRIGHT = {-2, -2},
    BOTTOMLEFT = {-2, -2},
    BOTTOMRIGHT = {0, 0},
    BOTTOM = {0, 0},
    TOP = {0, 0},
    LEFT = {2, 0},
    RIGHT = {2, 0},
    CENTER = {0, 0},
}
    local fonts = {}
    for k,v in pairs(ExPack.Fonts) do
        fonts[k] = k
    end
	
    local positions = {}
    for k,v in pairs(ExPack.PositionOffsets) do
        positions[k] = k
    end

local function createconfig()
	local options = {}

	options.type = "group"
	options.name = L["ExPack by |cFF66ccffGhordian |rOptions"]

	local function get(info)
		local ns,opt = string.split(".", info.arg)
		local val = ExPack.db.profile[ns][opt]
		--ExPack:Print("get", ns, opt, val)
		if type(val) == "table" then
			return unpack(val)
		else
			return val
		end
	end

	local function set(info, arg1, arg2, arg3, arg4)
		local ns,opt = string.split(".", info.arg)
		--ExPack:Print("set", ns, opt, arg1, arg2, arg3, arg4)
		if arg2 then
			local entry = ExPack.db.profile[ns][opt]
			entry[1] = arg1
			entry[2] = arg2
			entry[3] = arg3
			entry[4] = arg4
		else
			ExPack.db.profile[ns][opt] = arg1
		end

	end

	options.args = {}

	options.args.general = {
		type = 'group',
		order = 1,
		name = L["General Options"],
		get = get,
		set = set,
		args = {
			Enabled = {
				type = "toggle",
				name = L["Enable ExPack"],
				desc = L["Enables/Disables ExPack."],
				width = "double",
				get = function(info) return ExPack.db.profile.general.Enabled; end,
				set = function(info, v) ExPack.db.profile.general.Enabled = v; end,
				order = 1,
			},
			Section2 = {
				type = "header",
				name = L["Commands"],
				order = 40,
			},
			CMDs = {
				type = "description",
				name = L["/ep : Brings up this window."],
				fontSize = "medium",
				order = 41,
			},
			CMDs = {
				type = "description",
				name = "/ep minimap : will toggle the minimap icon.",
				fontSize = "medium",
				order = 42,
			},
			Section3 = {
				type = "header",
				name = "Synopsis",
				order = 50,
			},
			Synopsis = {
				type = "description",
				name = "Item ExPack Overlay addon.\r\n"..
						"\r\n" .. 
						"This addon will scan any item slot to apply a text overlay on will be shown the abbreviation of the "..
						"expansion pack the item belongs.\r\n\r\n" .. 
						"The text will be colored by the most popular color of the expansion, so it will be easily recognized "..
						"event when the background is dark.\r\n" ..					
						"\r\n" ..
						"TODO's\r\n"..
						" + Add more filter options.\r\n"..
						" + Add more visual options.\r\n"..
						" + Compatibility with many bag's addons like (but not only) Bagnon, AdiBags, OneBag3.\r\n"..
						"\r\n" ..
						"All item information is gathered from Wowhead.\r\n" ..					
						"\r\n"..
						"Thank you all for your feedback and suggestions!",
				fontSize = "medium",
				order = 51,
			},
			Section4 = {
				type = "header",
				name = L["Author"],
				order = 60,
			},
			About = {
				type = "description",
				name = "Hi, I'm Ghordian from EU-Mazrigos.\r\n\r\n" ..
						"I have been playing WoW since 2009 but never dare to write my own addon until now ;-)\r\n" ..
						"\r\n\r\n" ..
						"CREDITS: I do not own any of the images for the icons.\r\n" ..
						"I do not own the ItemVersion data table.\r\n\r\n"..
						"I want to thank my fellow guild members for being the first beta testers.\r\n",
				fontSize = "medium",
				order = 61,
			},
		},
	} -- End general block settings

	options.args.minimap = {
		type = 'group',
		order = 1,
		name = "Minimap",
		get = get,
		set = set,
		args = {
			Section1 = {
				type = "header",
				name = "Minimap",
				order = 1,
			},
			ErrorReporting = {
				type = "toggle",
				name = "Hide/show minimap button",
				desc = "Hide/show minimap button",
				width = "double",
				get = function(info) return ExPack.db.profile.minimap.hide; end,
				set = function(info, v) 
						ExPack.db.profile.minimap.hide = v; 
						ExPack:UpdateMinimapButton();
					end,
				order = 2,
			},
		}
	} -- End minimap
	
	options.args.filters = {
		type = 'group',
		order = 2,
		name = L["Filter Options"],
		get = get,
		set = set,
		args = {
			Section1 = {
				type = "header",
				name = L["Filters"],
				order = 1,
			},
			ShowAll = {
				type = "toggle",
				name = L["ShowAll"],
				desc = "Displays which expansion any item is from",
				width = "double",
				get = function(info) return ExPack.db.profile.filters.ShowAll; end,
				set = function(info, v) ExPack.db.profile.filters.ShowAll = v end,
				order = 2,
			},
			ShowReagents = {
				type = "toggle",
				name = L["ShowReagents"],
				desc = "Displays which expansion reagents is from",
				width = "double",
				get = function(info) return ExPack.db.profile.filters.ShowReagents; end,
				set = function(info, v) ExPack.db.profile.filters.ShowReagents = v end,
				disabled = function(info) return ExPack.db.profile.filters.ShowAll; end,
				order = 3,
			},
			ShowConsumable = {
				type = "toggle",
				name = L["ShowConsumable"],
				desc = "Displays which expansion reagents is from",
				width = "double",
				get = function(info) return ExPack.db.profile.filters.ShowConsumable; end,
				set = function(info, v) ExPack.db.profile.filters.ShowConsumable = v end,
				disabled = function(info) return ExPack.db.profile.filters.ShowAll; end,
				order = 4,
			},
			ShowTradegoods = {
				type = "toggle",
				name = L["ShowTradegoods"],
				desc = "Displays which expansion reagents is from",
				width = "double",
				get = function(info) return ExPack.db.profile.filters.ShowTradegoods; end,
				set = function(info, v) ExPack.db.profile.filters.ShowTradegoods = v end,
				disabled = function(info) return ExPack.db.profile.filters.ShowAll; end,
				order = 5,
			},
			ShowJunk = {
				type = "toggle",
				name = L["ShowJunk"],
				desc = "Displays which expansion reagents is from",
				width = "double",
				get = function(info) return ExPack.db.profile.filters.ShowJunk; end,
				set = function(info, v) ExPack.db.profile.filters.ShowJunk = v end,
				disabled = function(info) return ExPack.db.profile.filters.ShowAll; end,
				order = 6,
			},
			ShowQuestitem = {
				type = "toggle",
				name = L["ShowQuestitem"],
				desc = "Displays which expansion reagents is from",
				width = "double",
				get = function(info) return ExPack.db.profile.filters.ShowQuestitem; end,
				set = function(info, v) ExPack.db.profile.filters.ShowQuestitem = v end,
				disabled = function(info) return ExPack.db.profile.filters.ShowAll; end,
				order = 7,
			},
			ShowArmory  = {
				type = "toggle",
				name = L["ShowArmory"],
				desc = "Displays which expansion reagents is from",
				width = "double",
				get = function(info) return ExPack.db.profile.filters.ShowArmory; end,
				set = function(info, v) ExPack.db.profile.filters.ShowArmory = v end,
				disabled = function(info) return ExPack.db.profile.filters.ShowAll; end,
				order = 8,
			},
			Section2 = {
				type = "header",
				name = L["Banks"],
				order = 10,
			},
			ShowBanks = {
				type = "toggle",
				name = L["Enable Banks Only"],
				desc = "Displays which expansion item is from for any bank's slot",
				width = "double",
				get = function(info) return ExPack.db.profile.filters.ShowBanks; end,
				set = function(info, v) ExPack.db.profile.filters.ShowBanks = v; end,
				order = 11,
			},
			Section2 = {
				type = "header",
				name = L["Expansions"],
				order = 20,
			},
			AnyExpansion = {
				type = "toggle",
				name = L["Enable Any Expansion"],
				desc = "Displays which expansion item is from for any expansion pack",
				width = "double",
				get = function(info) return ExPack.db.profile.filters.AnyExpansion; end,
				set = function(info, v) ExPack.db.profile.filters.AnyExpansion = v; end,
				order = 21,
			},
			CurrentExpack = {
				type = "toggle",
				name = L["Enable Current Expack"],
				desc = "Displays which expansion item is from for current expansion pack",
				width = "double",
				get = function(info) return ExPack.db.profile.filters.CurrentExpack; end,
				set = function(info, v) ExPack.db.profile.filters.CurrentExpack = v; end,
				disabled = function(info) return ExPack.db.profile.filters.AnyExpansion; end,
				order = 22,
			},
			OldExpacks = {
				type = "toggle",
				name = L["Enable Old Expacks"],
				desc = "Displays which expansion item is from for old expansions pack",
				width = "double",
				get = function(info) return ExPack.db.profile.filters.OldExpacks; end,
				set = function(info, v) ExPack.db.profile.filters.OldExpacks = v; end,
				disabled = function(info) return ExPack.db.profile.filters.AnyExpansion; end,
				order = 23,
			},			
		},
	} -- End Filters block settings

	options.args.reports = {
		type = 'group',
		order = 3,
		name = L["Reports Options"],
		get = get,
		set = set,
		args = {
			Section1 = {
				type = "header",
				name = L["Reports"],
				order = 1,
			},
			ErrorReporting = {
				type = "toggle",
				name = L["Report errors to chat window"],
				desc = "Report errors to chat window",
				width = "double",
				get = function(info) return ExPack.db.profile.reports.ErrorReporting; end,
				set = function(info, v) ExPack.db.profile.reports.ErrorReporting = v; end,
				order = 2,
			},
			DebugTracing = {
				type = "toggle",
				name = L["Report debug tracing to chat window"],
				desc = "Report debug tracing to chat window",
				width = "double",
				get = function(info) return ExPack.db.profile.reports.DebugTracing; end,
				set = function(info, v) ExPack.db.profile.reports.DebugTracing = v; end,
				order = 3,
			},
			Section1 = {
				type = "header",
				name = L["ReloadUI"],
				order = 10,
			},
			ResetExPack = {
				type = "execute",
				name = L["Reset ExPack (Reloads UI after)"],
				desc = "Recommended after addon updates to correct missing profile settings.",
				width = "double",
				func = function()
					ExPackDB = nil;
					ExPackDB = {};
					ExPackDB.profile = ExPack.db.defaults.profile;
					ReloadUI();
				end,
				order = 11,
			},
		},
	} -- End reports block settings
	
	options.args.visual = {
		type = 'group',
		order = 4,
		name = L["Visual Options"],
		get = get,
		set = set,
		args = {
			VisualSection = {
				type = "header",
				name = L["Visual"],
				order = 1,
			},
			ShowCaption = {
				type = "toggle",
				name = L["Show Item Caption"],
				desc = "Displays colored caption",
				width = "double",
				get = function(info) return ExPack.db.profile.visual.ShowCaption; end,
				set = function(info, v) ExPack.db.profile.visual.ShowCaption = v 
						or not ExPack.db.profile.visual.ShowBorder; 
					end,
				order = 2,
			},
			ShowBorder = {
				type = "toggle",
				name = L["Show Item Border"],
				desc = "Displays colored outline border",
				width = "double",
				get = function(info) return ExPack.db.profile.visual.ShowBorder; end,
				set = function(info, v) ExPack.db.profile.visual.ShowBorder = v 
						or not ExPack.db.profile.visual.ShowCaption; 
					end,
				order = 3,
			},
			ExpansionSection = {
				type = "header",
				name = L["Expansions"],
				order = 4,
			},
			IconRecommend = {
				type = "description",
				name =  ExPack.majorToExpac[1].color.. " " .. ExPack.majorToExpac[1].shortName .. "|r - " ..ExPack.majorToExpac[1].canonName.."\r\n"..
						ExPack.majorToExpac[2].color.. " " .. ExPack.majorToExpac[2].shortName .. "|r - " ..ExPack.majorToExpac[2].canonName.."\r\n"..
						ExPack.majorToExpac[3].color.. " " .. ExPack.majorToExpac[3].shortName .. "|r - " ..ExPack.majorToExpac[3].canonName.."\r\n"..
						ExPack.majorToExpac[4].color.. " " .. ExPack.majorToExpac[4].shortName .. "|r - " ..ExPack.majorToExpac[4].canonName.."\r\n"..
						ExPack.majorToExpac[5].color.. " " .. ExPack.majorToExpac[5].shortName .. "|r - " ..ExPack.majorToExpac[5].canonName.."\r\n"..
						ExPack.majorToExpac[6].color.. " " .. ExPack.majorToExpac[6].shortName .. "|r - " ..ExPack.majorToExpac[6].canonName.."\r\n"..
						ExPack.majorToExpac[7].color.. " " .. ExPack.majorToExpac[7].shortName .. "|r - " ..ExPack.majorToExpac[7].canonName.."\r\n"..
						ExPack.majorToExpac[8].color.. " " .. ExPack.majorToExpac[8].shortName .. "|r - " ..ExPack.majorToExpac[8].canonName.."\r\n"..
						ExPack.majorToExpac[9].color.. " " .. ExPack.majorToExpac[9].shortName .. "|r - " ..ExPack.majorToExpac[9].canonName.."\r\n"..
						ExPack.majorToExpac[10].color.." " .. ExPack.majorToExpac[10].shortName.. "|r - " ..ExPack.majorToExpac[10].canonName.."\r\n",
				width = "full",
				fontSize = "medium",
				order = 5,
			},
			CaptionSection = {
				type = "header",
				name = "Caption",
				order = 6,
			},

			CaptionPosition = {
				type = "select",
				name = L["PositionName"],
				desc = L["PositionDesc"],
				width = "normal",
				style= "radio",
				values = positions,
				get = function(info) return ExPack.db.profile.visual.Position; end,
				set = function(info, v) 
						ExPack.db.profile.visual.Position = v
					end,
				order = 7,
			},

			CaptionFont = {
				type = "select",
				name = L["FontName"],
				desc = L["FontDesc"],
				width = "normal",
				style = "radio",
				values = fonts,
				get = function(info) return ExPack.db.profile.visual.Font; end,
				set = function(info, v) 
						ExPack.db.profile.visual.Font = v; 
					end,
				order = 8,
			},
		},
	}
	
	options.args.advanced = {
		type = 'group',
		order = 4,
		name = L["Advanced Options"],
		get = get,
		set = set,
		args = {
			Section1 = {
				type = "header",
				name = L["Advanced Options"],
				order = 1,
			},
			UseLibItemInfo = {
				type = "toggle",
				name = L["UseLibItemInfo"],
				desc = "Use UseLibItemInfo when ItemId is not found in current database",
				width = "double",
				get = function(info) return ExPack.db.profile.advanced.UseLibItemInfo; end,
				set = function(info, v) ExPack.db.profile.advanced.UseLibItemInfo = v; end,
				order = 2
			},
			Section1 = {
				type = "header",
				name = L["ReloadUI"],
				order = 10,
			},
			ResetExPack = {
				type = "execute",
				name = L["Reset ExPack (Reloads UI after)"],
				desc = "Recommended after addon updates to correct missing profile settings.",
				width = "double",
				func = function()
					ExPackDB = nil;
					ExPackDB = {};
					ExPackDB.profile = ExPack.db.defaults.profile;
					ReloadUI();
				end,
				order = 11,
			},
		},
	}
	
	return options
end

local config = LibStub("AceConfig-3.0")
local dialog = LibStub("AceConfigDialog-3.0")
local registered = false;

local options
local function createBlizzOptions()
	options = createconfig()

	-- General Options
	config:RegisterOptionsTable("ExPack-General", options.args.general)
	local blizzPanel = dialog:AddToBlizOptions("ExPack-General", options.args.general.name, "ExPack")
	
	-- Minimap Options
	config:RegisterOptionsTable("ExPack-Minimap", options.args.minimap)
	dialog:AddToBlizOptions("ExPack-Minimap", options.args.minimap.name, "ExPack")

	-- Filters Block Options
	config:RegisterOptionsTable("ExPack-Filters", options.args.filters)
	dialog:AddToBlizOptions("ExPack-Filters", options.args.filters.name, "ExPack")
	
	-- Visual Block Options
	config:RegisterOptionsTable("ExPack-Visual", options.args.visual)
	dialog:AddToBlizOptions("ExPack-Visual", options.args.visual.name, "ExPack")
	
	-- Reports Block Options
	config:RegisterOptionsTable("ExPack-Reports", options.args.reports)
	dialog:AddToBlizOptions("ExPack-Reports", options.args.reports.name, "ExPack")
	
	-- Advanced Block Options
	config:RegisterOptionsTable("ExPack-Advanced", options.args.advanced)
	dialog:AddToBlizOptions("ExPack-Advanced", options.args.advanced.name, "ExPack")
	
	return blizzPanel	
end

local aboutOptions = {
	type = "group",
	name = L["ExPack by |cFF66ccffGhordian |rOptions"],
	args = {
		version = {
			order = 1,
			type = "description",
			name = function() return "Version: ItemExPack-".. ExPack.version end,
		}
	},
}

local blizzPanel
function ExPack:CreateConfigPanels()
	config:RegisterOptionsTable("ExPack", aboutOptions)
	local aboutFrame = dialog:AddToBlizOptions("ExPack", "ExPack")
	if not registered then
		blizzPanel = createBlizzOptions()
		registered = true
	end
end
